"""Tests that create/sync/distribute/publish MANY rpm plugin repositories."""
import os
import re
import unittest

from pulp_smash import config
from pulp_smash.pulp3.bindings import PulpTestCase, monitor_task

from pulp_rpm.tests.functional.utils import (
    gen_rpm_client,
    skip_if,
)

from pulpcore.client.pulp_rpm import (
    DistributionsRpmApi,
    PublicationsRpmApi,
    RemotesRpmApi,
    RepositoriesRpmApi,
    RpmRepositorySyncURL,
    RpmRpmPublication,
)


# 'export FIPS_WORKFLOW=anything"' to run this suite
@unittest.skipIf(
    not os.environ.get("FIPS_WORKFLOW", None),
    "This is a SIX HOUR test suit - run only when you're sure it's what you need",
)
class FipsRemotesTestCase(PulpTestCase):
    """
    Tests a large number of repositories for the full RPM workflow (create/sync/distribute/publish).

    Primarily useful for testing a wide range of 'real' repositories in the face of global
    issues like FIPS-enabled systems. The list of repositories of interest is taken from
    https://pulp.plan.io/issues/7537

    Because it is a long-running intensive set of tests, the entire test-class is skipped
    unless an environment-variable FIPS_WORKFLOW is set.

    For Red Hat CDN repositories, assumes that appropriate certificates are stored in the
    environment variables CDN_CA_CERT, CDN_CLIENT_CERT, and CDN_CLIENT_KEY.

    Tests as implemented use on_demand syncing. Running the entire test suit takes ~6 hours.
    Switching to immediate would take much longer, and require A LOT of disk space.

    NOTE: We check only for successful-completion of the various steps. It is 'assumed' that,
    when these tests run, we're already confident that the create/sync/distribute/publish
    path is reliable; any failures here would be due to specific data-issues, or configuration
    (e.g., FIPS realities and repository-checksum-realities interfering with each other).
    """

    @staticmethod
    def _name_from_url(url):
        """Converts a remote-url into a string suitable for name-fields."""
        # https://cdn.redhat.com/content/dist/rhel/server/6/6Server/x86_64/extras/os
        # cdn_redhat_com_content_dist_rhel_server_6_6Server_x86_64_extras_os

        # drop trailing slash
        rstr = url.rstrip("/")

        # drop protocol
        if rstr.startswith("https://"):
            rstr = rstr[8:]
        elif rstr.startswith("http://"):
            rstr = rstr[7:]

        # convert ./- into underscore
        rstr = re.sub("[.\-/]", "_", rstr)  # noqa
        return rstr

    @classmethod
    def setUpClass(cls):
        """Create class-wide variables."""
        cls.cfg = config.get_config()
        cls.client = gen_rpm_client()
        cls.repo_api = RepositoriesRpmApi(cls.client)
        cls.remote_api = RemotesRpmApi(gen_rpm_client())
        cls.publications = PublicationsRpmApi(cls.client)
        cls.distributions = DistributionsRpmApi(cls.client)

        if (
            os.environ.get("CDN_CLIENT_CERT", None)
            and os.environ.get("CDN_CLIENT_KEY", None)
            and os.environ.get("CDN_CA_CERT", None)
        ):
            # strings have escaped newlines from environmental variable
            cls.cdn_client_cert = os.environ["CDN_CLIENT_CERT"].replace("\\n", "\n")
            cls.cdn_client_key = os.environ["CDN_CLIENT_KEY"].replace("\\n", "\n")
            cls.cdn_ca_cert = os.environ["CDN_CA_CERT"].replace("\\n", "\n")
        else:
            cls.cdn_client_cert = None

    def _create_remote_for(self, url):
        name = self._name_from_url(url)
        body = {"name": name, "url": url, "policy": "on_demand"}
        # Red Hat repos are at cdn.redhat.com, and require cert-auth for access
        if name.startswith("cdn_"):
            self.assertIsNotNone(self.cdn_client_cert, "Can't test CDN repositories, missing certs")
            # need cert-access
            body["ca_cert"] = self.cdn_ca_cert
            body["client_cert"] = self.cdn_client_cert
            body["client_key"] = self.cdn_client_key
        remote = self.remote_api.create(body)
        return remote

    def _create_repo_for(self, url):
        name = self._name_from_url(url)
        body = {"name": name}
        repo = self.repo_api.create(body)
        return repo

    def _create_publication_for(self, repo):
        publish_data = RpmRpmPublication(repository=repo.pulp_href)
        publish_response = self.publications.create(publish_data)
        created_resources = monitor_task(publish_response.task).created_resources
        self.assertIsNotNone(created_resources)
        self.assertTrue(len(created_resources) > 0)
        publication_href = created_resources[0]
        return publication_href

    def _create_distribution_for(self, name, pub):
        body = {"base_path": name, "name": name, "publication": pub}
        distribution_response = self.distributions.create(body)
        created_resources = monitor_task(distribution_response.task).created_resources
        self.assertIsNotNone(created_resources)
        self.assertTrue(len(created_resources) > 0)
        distribution = self.distributions.read(created_resources[0])
        return distribution

    def _do_test(self, url):
        # Convert a url into a name-string
        name = self._name_from_url(url)

        # Create a repo
        repo = self._create_repo_for(url)
        self.assertIsNotNone(repo)
        self.addCleanup(self.repo_api.delete, repo.pulp_href)

        # Create a remote
        remote = self._create_remote_for(url)
        self.assertIsNotNone(remote)
        self.addCleanup(self.remote_api.delete, remote.pulp_href)

        # Sync the repo using the remote
        repository_sync_data = RpmRepositorySyncURL(remote=remote.pulp_href)
        sync_response = self.repo_api.sync(repo.pulp_href, repository_sync_data)
        monitor_task(sync_response.task)

        # Publish the result
        publication_href = self._create_publication_for(repo)
        self.assertIsNotNone(publication_href)
        self.addCleanup(self.publications.delete, publication_href)

        # Distribute the published version
        distribution = self._create_distribution_for(name, publication_href)
        self.assertIsNotNone(distribution)
        self.addCleanup(self.distributions.delete, distribution.pulp_href)

    @skip_if(bool, "cdn_client_cert", False)
    def test_000_cdn_redhat_com_content_dist_rhel_server_6_6Server_x86_64_extras_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6Server/x86_64/extras/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_001_cdn_redhat_com_content_dist_rhel_server_6_6Server_x86_64_optional_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6Server/x86_64/optional/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_002_cdn_redhat_com_content_dist_rhel_server_6_6Server_x86_64_supplementary_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6Server/x86_64/supplementary/os"
        self._do_test(url)

    @unittest.skip("fails with 'NoneType' object is not subscriptable?")
    @skip_if(bool, "cdn_client_cert", False)
    def test_003_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_004_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_extras_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/extras/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_005_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_optional_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/optional/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_006_cdn_redhat_com_content_dist_rhel_server_6_6Server_x86_64_rhscl_1_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6Server/x86_64/rhscl/1/os"
        self._do_test(url)

    def test_007_mirror_centos_org_centos_7_7_extras_x86_64(self):  # noqa D102
        url = "http://mirror.centos.org/centos-7/7/extras/x86_64/"
        self._do_test(url)

    def test_008_mirror_centos_org_centos_7_7_sclo_x86_64_sclo(self):  # noqa D102
        url = "http://mirror.centos.org/centos-7/7/sclo/x86_64/sclo/"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_009_cdn_redhat_com_content_eus_rhel_server_6_6_6_x86_64_optional_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/eus/rhel/server/6/6.6/x86_64/optional/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_010_cdn_redhat_com_content_dist_rhel_server_7_7_7_x86_64_kickstart(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7.7/x86_64/kickstart"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_011_cdn_redhat_com_content_dist_rhel8_8_0_x86_64_baseos_kickstart(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel8/8.0/x86_64/baseos/kickstart"
        self._do_test(url)

    def test_012_mirrors_kernel_org_fedora_epel_7_x86_64(self):  # noqa D102
        url = "https://mirrors.kernel.org/fedora-epel/7/x86_64/"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_013_cdn_redhat_com_content_dist_rhel_server_6_6_7_x86_64_kickstart(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6.7/x86_64/kickstart"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_014_cdn_redhat_com_content_eus_rhel_server_6_6_6_x86_64_rhscl_1_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/eus/rhel/server/6/6.6/x86_64/rhscl/1/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_015_cdn_redhat_com_content_eus_rhel_server_6_6_6_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/eus/rhel/server/6/6.6/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_016_cdn_redhat_com_content_dist_rhel_server_7_7_3_x86_64_kickstart(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7.3/x86_64/kickstart"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_017_cdn_redhat_com_content_dist_rhel8_8_0_x86_64_appstream_kickstart(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel8/8.0/x86_64/appstream/kickstart"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_018_cdn_redhat_com_content_eus_rhel_server_7_7_3_x86_64_optional_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.3/x86_64/optional/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_019_cdn_redhat_com_content_eus_rhel_server_7_7_3_x86_64_supplementary_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.3/x86_64/supplementary/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_020_cdn_redhat_com_content_eus_rhel_server_7_7_3_x86_64_rhscl_1_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.3/x86_64/rhscl/1/os"
        self._do_test(url)

    def test_021_mirror_centos_org_centos_6_6_os_x86_64(self):  # noqa D102
        # centos-6 has been archived, 6.10 is the last one
        url = "http://vault.centos.org/6.10/os/x86_64/"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_022_cdn_redhat_com_content_dist_rhel_server_6_6Server_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6Server/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_023_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_ansible_2_5_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/ansible/2.5/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_024_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_rhgs_server_nfs_3_1_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/rhgs-server-nfs/3.1/os"  # noqa E501
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_025_cdn_redhat_com_content_dist_rhel_server_7_7_6_x86_64_kickstart(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7.6/x86_64/kickstart"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_026_cdn_redhat_com_content_dist_rhel_workstation_7_7_5_x86_64_kickstart(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/workstation/7/7.5/x86_64/kickstart"
        self._do_test(url)

    def test_027_mirrors_kernel_org_fedora_epel_8_Everything_x86_64(self):  # noqa D102
        url = "https://mirrors.kernel.org/fedora-epel/8/Everything/x86_64/"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_028_cdn_redhat_com_content_dist_rhel_workstation_7_7Workstation_x86_64_insights_3_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/workstation/7/7Workstation/x86_64/insights/3/os"  # noqa E501
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_029_cdn_redhat_com_content_dist_rhel_workstation_7_7Workstation_x86_64_optional_os(  # noqa D102
        self,
    ):
        url = (
            "https://cdn.redhat.com/content/dist/rhel/workstation/7/7Workstation/x86_64/optional/os"
        )
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_030_cdn_redhat_com_content_dist_rhel_workstation_7_7Workstation_x86_64_rh_common_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/workstation/7/7Workstation/x86_64/rh-common/os"  # noqa E501
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_031_cdn_redhat_com_content_dist_rhel_workstation_7_7Workstation_x86_64_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/workstation/7/7Workstation/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_032_cdn_redhat_com_content_dist_rhel_workstation_7_7Workstation_x86_64_extras_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/workstation/7/7Workstation/x86_64/extras/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_033_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_rh_gluster_samba_3_1_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/rh-gluster-samba/3.1/os"  # noqa E501
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_034_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_rhgs_server_3_1_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/rhgs-server/3.1/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_035_cdn_redhat_com_content_dist_rhel_server_6_6_10_x86_64_kickstart(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6.10/x86_64/kickstart"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_036_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_rhgs_nagios_3_1_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/rhgs-nagios/3.1/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_037_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_rhscon_agent_2_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/rhscon-agent/2/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_038_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_rhscon_installer_2_os(  # noqa D102
        self,
    ):
        url = (
            "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/rhscon-installer/2/os"
        )
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_039_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_rhscon_main_2_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/rhscon-main/2/os"
        self._do_test(url)

    def test_040_mirror_centos_org_centos_6_6_updates_x86_64(self):  # noqa D102
        # centos-6 has been archived, 6.10 is the last one
        url = "http://vault.centos.org/6.10/updates/x86_64/"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_041_cdn_redhat_com_content_dist_rhel_server_6_6Server_x86_64_rhs_client_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6Server/x86_64/rhs-client/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_042_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_rhs_client_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/rhs-client/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_043_mirror_centos_org_centos_7_7_sclo_x86_64_rh(self):  # noqa D102
        url = "http://mirror.centos.org/centos-7/7/sclo/x86_64/rh/"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_044_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_supplementary_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/supplementary/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_045_cdn_redhat_com_content_dist_rhel_workstation_7_7Workstation_x86_64_rhscl_1_os(  # noqa D102
        self,
    ):
        url = (
            "https://cdn.redhat.com/content/dist/rhel/workstation/7/7Workstation/x86_64/rhscl/1/os"
        )
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_046_cdn_redhat_com_content_dist_rhel_workstation_7_7_6_x86_64_kickstart(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/workstation/7/7.6/x86_64/kickstart"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_047_mirror_centos_org_centos_7_7_updates_x86_64(self):  # noqa D102
        url = "http://mirror.centos.org/centos-7/7/updates/x86_64/"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_048_cdn_redhat_com_content_dist_rhel_server_6_6_8_x86_64_kickstart(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6.8/x86_64/kickstart"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_049_cdn_redhat_com_content_dist_rhel_server_6_6_9_x86_64_kickstart(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6.9/x86_64/kickstart"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_050_cdn_redhat_com_content_eus_rhel_server_6_6_6_x86_64_sat_tools_6_2_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/eus/rhel/server/6/6.6/x86_64/sat-tools/6.2/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_051_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_rhscl_1_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/rhscl/1/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_052_mirror_centos_org_centos_7_7_os_x86_64(self):  # noqa D102
        url = "http://mirror.centos.org/centos-7/7/os/x86_64/"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_053_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_ansible_2_7_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/ansible/2.7/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_054_cdn_redhat_com_content_eus_rhel_server_7_7_3_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.3/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_055_cdn_redhat_com_content_eus_rhel_server_7_7_6_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.6/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_056_cdn_redhat_com_content_dist_rhel_server_7_7_4_x86_64_kickstart(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7.4/x86_64/kickstart"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_057_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_sat_maintenance_6_os(  # noqa D102
        self,
    ):
        url = (
            "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/sat-maintenance/6/os"
        )
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_058_cdn_redhat_com_content_dist_rhel_server_7_7_5_x86_64_kickstart(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7.5/x86_64/kickstart"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_059_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_rhgs_server_bigdata_3_1_os(  # noqa E501
        self,
    ):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/rhgs-server-bigdata/3.1/os"  # noqa E501
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_060_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_rhgs_server_splunk_3_1_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/rhgs-server-splunk/3.1/os"  # noqa E501
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_061_cdn_redhat_com_content_dist_rhel_workstation_7_7Workstation_x86_64_supplementary_os(  # noqa E501
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/workstation/7/7Workstation/x86_64/supplementary/os"  # noqa E501
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_062_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_ansible_2_6_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/ansible/2.6/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_063_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_dotnet_1_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/dotnet/1/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_064_cdn_redhat_com_content_dist_rhel_server_6_6_10_x86_64_optional_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6.10/x86_64/optional/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_065_cdn_redhat_com_content_eus_rhel_server_7_7Server_x86_64_sat_tools_6_5_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7Server/x86_64/sat-tools/6.5/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_066_cdn_redhat_com_content_eus_rhel_server_7_7_5_x86_64_sat_tools_6_5_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.5/x86_64/sat-tools/6.5/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_067_cdn_redhat_com_content_dist_rhel_server_7_7_7_x86_64_optional_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7.7/x86_64/optional/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_068_cdn_redhat_com_content_dist_rhel_server_7_7_4_x86_64_optional_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7.4/x86_64/optional/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_069_cdn_redhat_com_content_eus_rhel_server_6_6_7_x86_64_supplementary_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/eus/rhel/server/6/6.7/x86_64/supplementary/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_070_cdn_redhat_com_content_eus_rhel_server_6_6_7_x86_64_optional_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/eus/rhel/server/6/6.7/x86_64/optional/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_071_cdn_redhat_com_content_eus_rhel_server_6_6_7_x86_64_rhscl_1_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/eus/rhel/server/6/6.7/x86_64/rhscl/1/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_072_cdn_redhat_com_content_eus_rhel_server_7_7_5_x86_64_rhscl_1_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.5/x86_64/rhscl/1/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_073_cdn_redhat_com_content_eus_rhel_server_7_7_5_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.5/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_074_cdn_redhat_com_content_dist_rhel_server_6_6_7_x86_64_optional_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6.7/x86_64/optional/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_075_cdn_redhat_com_content_dist_rhel_server_6_6_10_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6.10/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_076_cdn_redhat_com_content_dist_rhel_server_6_6_8_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6.8/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_077_cdn_redhat_com_content_dist_rhel_server_6_6_6_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6.6/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_078_cdn_redhat_com_content_eus_rhel_server_6_6_7_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/eus/rhel/server/6/6.7/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_079_cdn_redhat_com_content_eus_rhel_server_7_7_5_x86_64_supplementary_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.5/x86_64/supplementary/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_080_cdn_redhat_com_content_eus_rhel_server_7_7_5_x86_64_sat_tools_6_4_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.5/x86_64/sat-tools/6.4/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_081_cdn_redhat_com_content_eus_rhel_server_7_7_5_x86_64_optional_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.5/x86_64/optional/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_082_cdn_redhat_com_content_eus_rhel_server_7_7_3_x86_64_sat_tools_6_4_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.3/x86_64/sat-tools/6.4/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_083_cdn_redhat_com_content_dist_rhel8_8_x86_64_appstream_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel8/8/x86_64/appstream/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_084_cdn_redhat_com_content_dist_rhel8_8_x86_64_baseos_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel8/8/x86_64/baseos/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_085_cdn_redhat_com_content_dist_rhel8_8_x86_64_supplementary_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel8/8/x86_64/supplementary/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_086_cdn_redhat_com_content_dist_rhel8_8_x86_64_baseos_kickstart(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel8/8/x86_64/baseos/kickstart"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_087_cdn_redhat_com_content_dist_rhel8_8_x86_64_appstream_kickstart(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel8/8/x86_64/appstream/kickstart"
        self._do_test(url)

    def test_088_mirrors_kernel_org_fedora_epel_6Server_x86_64(self):  # noqa D102
        # epel 6 has been archived
        url = "https://archives.fedoraproject.org/pub/archive/epel/6/x86_64/"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_089_cdn_redhat_com_content_dist_rhel_server_7_7_6_x86_64_optional_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7.6/x86_64/optional/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_090_cdn_redhat_com_content_dist_rhel_server_7_7_3_x86_64_optional_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7.3/x86_64/optional/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_091_cdn_redhat_com_content_dist_rhel_server_6_6_9_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6.9/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_092_cdn_redhat_com_content_dist_rhel_server_6_6_8_x86_64_optional_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6.8/x86_64/optional/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_093_cdn_redhat_com_content_dist_rhel_server_6_6_7_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6.7/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_094_cdn_redhat_com_content_dist_rhel_server_6_6_9_x86_64_optional_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6.9/x86_64/optional/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_095_cdn_redhat_com_content_dist_rhel_server_7_7_5_x86_64_optional_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7.5/x86_64/optional/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_096_cdn_redhat_com_content_dist_rhel_server_7_7_2_x86_64_optional_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7.2/x86_64/optional/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_097_cdn_redhat_com_content_dist_rhel_server_7_7_6_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7.6/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_098_cdn_redhat_com_content_dist_rhel_server_7_7_5_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7.5/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_099_cdn_redhat_com_content_dist_rhel_server_7_7_3_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7.3/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_100_cdn_redhat_com_content_eus_rhel_server_7_7_6_x86_64_sat_tools_6_5_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.6/x86_64/sat-tools/6.5/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_101_cdn_redhat_com_content_eus_rhel_server_7_7_6_x86_64_optional_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.6/x86_64/optional/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_102_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_sat_capsule_6_6_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/sat-capsule/6.6/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_103_cdn_redhat_com_content_dist_rhel_server_6_6Server_x86_64_sat_tools_6_6_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6Server/x86_64/sat-tools/6.6/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_104_cdn_redhat_com_content_dist_layered_rhel8_x86_64_sat_tools_6_6_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/layered/rhel8/x86_64/sat-tools/6.6/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_105_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_ansible_2_8_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/ansible/2.8/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_106_cdn_redhat_com_content_eus_rhel_server_7_7_6_x86_64_sat_tools_6_6_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.6/x86_64/sat-tools/6.6/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_107_cdn_redhat_com_content_dist_rhel_server_7_7_7_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7.7/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_108_cdn_redhat_com_content_dist_rhel8_8_1_x86_64_appstream_kickstart(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel8/8.1/x86_64/appstream/kickstart"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_109_cdn_redhat_com_content_dist_rhel_server_7_7_4_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7.4/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_110_cdn_redhat_com_content_dist_rhel_server_7_7_2_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7.2/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_111_cdn_redhat_com_content_eus_rhel_server_7_7_6_x86_64_supplementary_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.6/x86_64/supplementary/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_112_cdn_redhat_com_content_eus_rhel_server_7_7_6_x86_64_rhscl_1_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.6/x86_64/rhscl/1/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_113_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_sat_tools_6_6_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/sat-tools/6.6/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_114_cdn_redhat_com_content_eus_rhel_server_7_7_5_x86_64_sat_tools_6_6_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.5/x86_64/sat-tools/6.6/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_115_cdn_redhat_com_content_dist_rhel8_8_1_x86_64_baseos_kickstart(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel8/8.1/x86_64/baseos/kickstart"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_116_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_highavailability_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/highavailability/os"
        self._do_test(url)

    def test_117_packages_vmware_com_tools_releases_10_3_5_rhel6_x86_64(self):  # noqa D102
        url = "https://packages.vmware.com/tools/releases/10.3.5/rhel6/x86_64/"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_118_cdn_redhat_com_content_dist_rhel_server_7_7_8_x86_64_kickstart(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7.8/x86_64/kickstart"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_119_cdn_redhat_com_content_eus_rhel_server_7_7_7_x86_64_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.7/x86_64/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_120_cdn_redhat_com_content_eus_rhel_server_7_7_7_x86_64_supplementary_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.7/x86_64/supplementary/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_121_cdn_redhat_com_content_eus_rhel_server_7_7_7_x86_64_rhscl_1_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.7/x86_64/rhscl/1/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_122_cdn_redhat_com_content_eus_rhel_server_7_7_7_x86_64_optional_os(self):  # noqa D102
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.7/x86_64/optional/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_123_cdn_redhat_com_content_eus_rhel_server_7_7_7_x86_64_sat_tools_6_6_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/eus/rhel/server/7/7.7/x86_64/sat-tools/6.6/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_124_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_sat_capsule_6_7_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/sat-capsule/6.7/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_125_cdn_redhat_com_content_dist_rhel_server_6_6Server_x86_64_sat_tools_6_7_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/6/6Server/x86_64/sat-tools/6.7/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_126_cdn_redhat_com_content_dist_rhel_server_7_7Server_x86_64_sat_tools_6_7_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel/server/7/7Server/x86_64/sat-tools/6.7/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_127_cdn_redhat_com_content_dist_layered_rhel8_x86_64_sat_tools_6_7_os(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/layered/rhel8/x86_64/sat-tools/6.7/os"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_128_cdn_redhat_com_content_dist_rhel8_8_2_x86_64_appstream_kickstart(  # noqa D102
        self,
    ):
        url = "https://cdn.redhat.com/content/dist/rhel8/8.2/x86_64/appstream/kickstart"
        self._do_test(url)

    @skip_if(bool, "cdn_client_cert", False)
    def test_129_cdn_redhat_com_content_dist_rhel8_8_2_x86_64_baseos_kickstart(self):  # noqa D102
        url = "https://cdn.redhat.com/content/dist/rhel8/8.2/x86_64/baseos/kickstart"
        self._do_test(url)

    def test_130_mirror_centos_org_centos_8_8_BaseOS_x86_64_os(self):  # noqa D102
        url = "http://mirror.centos.org/centos-8/8/BaseOS/x86_64/os/"
        self._do_test(url)

    def test_131_mirror_centos_org_centos_8_8_AppStream_x86_64_os(self):  # noqa D102
        url = "http://mirror.centos.org/centos-8/8/AppStream/x86_64/os/"
        self._do_test(url)



# Generated by Django 2.2.16 on 2020-10-09 14:58

from django.db import migrations, models


def check_checksum(apps, schema_editor):
    checksum_model = apps.get_model("rpm", "Checksum")
    repository_model = apps.get_model("rpm", "RpmRepository")
    q_bad_dist_tree = checksum_model.objects.filter(checksum=None)
    if q_bad_dist_tree:
        dist_tree_pks = {
            dt.distribution_tree.pk
            for dt in q_bad_dist_tree
        }
        repository_list = {
            f"/pulp/api/v3/repositories/rpm/rpm/{repo.pk}"
            for repo in repository_model.objects.filter(content__in=dist_tree_pks)
        }
        raise ValueError(
            "Your Pulp instance contains DistributionTree content units with null checksums. "
            "They were not properly validated before this update. Please remove the affected "
            f"DistributionTrees content from your repositories {repository_list} by removing "
            "old repository versions and running orphan cleanup afterwards. "
            "If you need to get removed content back, please re-sync corresponding repositories."
        )


class Migration(migrations.Migration):

    dependencies = [
        ('rpm', '0026_add_gpgcheck_options'),
    ]

    operations = [
        migrations.RunPython(check_checksum),
        migrations.AlterField(
            model_name='checksum',
            name='checksum',
            field=models.CharField(max_length=128),
            preserve_default=False,
        ),
    ]



# Generated by Django 2.2.13 on 2020-07-10 09:41

from django.db import migrations, models


def assign_relative_path(apps, schema_editor):
    repo_metadata = apps.get_model('rpm', 'RepoMetadataFile')
    for metadata in repo_metadata.objects.all():
        metadata.relative_path = metadata.contentartifact_set.first().relative_path
        metadata.save()


class Migration(migrations.Migration):

    dependencies = [
        ('rpm', '0014_rpmrepository_package_retention_policy'),
    ]

    operations = [
        migrations.AddField(
            model_name='repometadatafile',
            name='relative_path',
            field=models.TextField(default=''),
            preserve_default=False,
        ),
        migrations.AlterUniqueTogether(
            name='repometadatafile',
            unique_together={('data_type', 'checksum', 'relative_path')},
        ),
        migrations.RunPython(assign_relative_path)
    ]



from .advisory import (  # noqa
    MinimalUpdateRecordSerializer,
    UpdateCollectionSerializer,
    UpdateRecordSerializer,
)
from .comps import (  # noqa
    PackageCategorySerializer,
    PackageEnvironmentSerializer,
    PackageGroupSerializer,
    PackageLangpacksSerializer,
)
from .custom_metadata import RepoMetadataFileSerializer  # noqa
from .distribution import (  # noqa
    AddonSerializer,
    ChecksumSerializer,
    DistributionTreeSerializer,
    ImageSerializer,
    VariantSerializer,
)
from .modulemd import ModulemdSerializer, ModulemdDefaultsSerializer  # noqa
from .package import PackageSerializer, MinimalPackageSerializer  # noqa
from .repository import (  # noqa
    CopySerializer,
    RpmDistributionSerializer,
    RpmPublicationSerializer,
    RpmRemoteSerializer,
    UlnRemoteSerializer,
    RpmRepositorySerializer,
    RpmRepositorySyncURLSerializer,
)



# coding=utf-8
"""Tests that sync file plugin repositories."""
import unittest

from pulp_smash import config
from pulp_smash.pulp3.bindings import monitor_task
from pulp_smash.pulp3.utils import gen_repo

from pulp_rpm.tests.functional.utils import gen_rpm_client, gen_rpm_remote
from pulp_rpm.tests.functional.utils import set_up_module as setUpModule  # noqa:F401

from pulpcore.client.pulp_rpm import (
    ContentPackagesApi,
    DistributionsRpmApi,
    PublicationsRpmApi,
    RepositoriesRpmApi,
    RpmRepositorySyncURL,
    RemotesRpmApi,
)


class AutoPublishDistributeTestCase(unittest.TestCase):
    """Test auto-publish and auto-distribution."""

    @classmethod
    def setUpClass(cls):
        """Create class-wide variables."""
        cls.cfg = config.get_config()
        cls.client = gen_rpm_client()

        cls.content_api = ContentPackagesApi(cls.client)
        cls.repo_api = RepositoriesRpmApi(cls.client)
        cls.remote_api = RemotesRpmApi(cls.client)
        cls.publications_api = PublicationsRpmApi(cls.client)
        cls.distributions_api = DistributionsRpmApi(cls.client)

    def setUp(self):
        """Create remote, repo, publish settings, and distribution."""
        self.remote = self.remote_api.create(gen_rpm_remote())
        self.repo = self.repo_api.create(gen_repo(autopublish=True, sqlite_metadata=True))
        response = self.distributions_api.create(
            {"name": "foo", "base_path": "bar/foo", "repository": self.repo.pulp_href}
        )
        distribution_href = monitor_task(response.task).created_resources[0]
        self.distribution = self.distributions_api.read(distribution_href)

    def tearDown(self):
        """Clean up."""
        monitor_task(self.repo_api.delete(self.repo.pulp_href).task)
        monitor_task(self.remote_api.delete(self.remote.pulp_href).task)
        monitor_task(self.distributions_api.delete(self.distribution.pulp_href).task)

    def test_01_sync(self):
        """Assert that syncing the repository triggers auto-publish and auto-distribution."""
        self.assertEqual(self.publications_api.list().count, 0)
        self.assertTrue(self.distribution.publication is None)

        # Sync the repository.
        repository_sync_data = RpmRepositorySyncURL(remote=self.remote.pulp_href)
        sync_response = self.repo_api.sync(self.repo.pulp_href, repository_sync_data)
        task = monitor_task(sync_response.task)
        self.distribution = self.distributions_api.read(self.distribution.pulp_href)

        # Check that all the appropriate resources were created
        self.assertGreater(len(task.created_resources), 1)
        self.assertEqual(self.publications_api.list().count, 1)
        self.assertTrue(self.distribution.publication is not None)
        self.assertTrue(self.distribution.publication in task.created_resources)

        # Check that the publish settings were used
        publication = self.publications_api.read(self.distribution.publication)
        self.assertEqual(publication.sqlite_metadata, True)

        # Sync the repository again. Since there should be no new repository version, there
        # should be no new publications or distributions either.
        sync_response = self.repo_api.sync(self.repo.pulp_href, repository_sync_data)
        task = monitor_task(sync_response.task)

        self.assertEqual(len(task.created_resources), 0)
        self.assertEqual(self.publications_api.list().count, 1)

    def test_02_modify(self):
        """Assert that modifying the repository triggers auto-publish and auto-distribution."""
        self.assertEqual(self.publications_api.list().count, 0)
        self.assertTrue(self.distribution.publication is None)

        # Modify the repository by adding a coment unit
        content = self.content_api.list().results[0].pulp_href

        modify_response = self.repo_api.modify(
            self.repo.pulp_href, {"add_content_units": [content]}
        )
        task = monitor_task(modify_response.task)
        self.distribution = self.distributions_api.read(self.distribution.pulp_href)

        # Check that all the appropriate resources were created
        self.assertGreater(len(task.created_resources), 1)
        self.assertEqual(self.publications_api.list().count, 1)
        self.assertTrue(self.distribution.publication is not None)
        self.assertTrue(self.distribution.publication in task.created_resources)

        # Check that the publish settings were used
        publication = self.publications_api.read(self.distribution.publication)
        self.assertEqual(publication.sqlite_metadata, True)



# Generated by Django 2.2.15 on 2020-09-04 16:17

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('rpm', '0023_increase_distribution_release_short'),
    ]

    operations = [
        migrations.AlterField(
            model_name='variant',
            name='repository',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='variants', to='core.Repository'),
        ),
    ]



from types import SimpleNamespace

CHECKSUM_TYPES = SimpleNamespace(
    UNKNOWN="unknown",
    MD5="md5",
    SHA="sha1",  # compatibility nickname from original createrepo
    SHA1="sha1",
    SHA224="sha224",
    SHA256="sha256",
    SHA384="sha384",
    SHA512="sha512",
)

# The same as above, but in a format that choice fields can use
CHECKSUM_CHOICES = (
    (CHECKSUM_TYPES.UNKNOWN, CHECKSUM_TYPES.UNKNOWN),
    (CHECKSUM_TYPES.MD5, CHECKSUM_TYPES.MD5),
    (CHECKSUM_TYPES.SHA, CHECKSUM_TYPES.SHA1),
    (CHECKSUM_TYPES.SHA1, CHECKSUM_TYPES.SHA1),
    (CHECKSUM_TYPES.SHA224, CHECKSUM_TYPES.SHA224),
    (CHECKSUM_TYPES.SHA256, CHECKSUM_TYPES.SHA256),
    (CHECKSUM_TYPES.SHA384, CHECKSUM_TYPES.SHA384),
    (CHECKSUM_TYPES.SHA512, CHECKSUM_TYPES.SHA512),
)

ALLOWED_CHECKSUM_ERROR_MSG = """Checksum must be one of the allowed checksum types.
You can adjust these with the 'ALLOWED_CONTENT_CHECKSUMS' setting."""

CR_PACKAGE_ATTRS = SimpleNamespace(
    ARCH="arch",
    CHANGELOGS="changelogs",
    CHECKSUM_TYPE="checksum_type",
    CONFLICTS="conflicts",
    DESCRIPTION="description",
    ENHANCES="enhances",
    EPOCH="epoch",
    FILES="files",
    LOCATION_BASE="location_base",
    LOCATION_HREF="location_href",
    NAME="name",
    OBSOLETES="obsoletes",
    PKGID="pkgId",
    PROVIDES="provides",
    RECOMMENDS="recommends",
    RELEASE="release",
    REQUIRES="requires",
    RPM_BUILDHOST="rpm_buildhost",
    RPM_GROUP="rpm_group",
    RPM_HEADER_END="rpm_header_end",
    RPM_HEADER_START="rpm_header_start",
    RPM_LICENSE="rpm_license",
    RPM_PACKAGER="rpm_packager",
    RPM_SOURCERPM="rpm_sourcerpm",
    RPM_VENDOR="rpm_vendor",
    SIZE_ARCHIVE="size_archive",
    SIZE_INSTALLED="size_installed",
    SIZE_PACKAGE="size_package",
    SUGGESTS="suggests",
    SUMMARY="summary",
    SUPPLEMENTS="supplements",
    TIME_BUILD="time_build",
    TIME_FILE="time_file",
    URL="url",
    VERSION="version",
)

PACKAGE_REPODATA = ["primary", "filelists", "other"]
PACKAGE_DB_REPODATA = ["primary_db", "filelists_db", "other_db"]
UPDATE_REPODATA = ["updateinfo"]
MODULAR_REPODATA = ["modules"]
COMPS_REPODATA = ["group"]
SKIP_TYPES = ["srpm"]

CR_UPDATE_RECORD_ATTRS = SimpleNamespace(
    ID="id",
    UPDATED_DATE="updated_date",
    DESCRIPTION="description",
    ISSUED_DATE="issued_date",
    FROMSTR="fromstr",
    STATUS="status",
    TITLE="title",
    SUMMARY="summary",
    VERSION="version",
    TYPE="type",
    SEVERITY="severity",
    SOLUTION="solution",
    RELEASE="release",
    RIGHTS="rights",
    PUSHCOUNT="pushcount",
    REBOOT_SUGGESTED="reboot_suggested",
)

CR_UPDATE_COLLECTION_ATTRS = SimpleNamespace(NAME="name", SHORTNAME="shortname", MODULE="module")

CR_UPDATE_COLLECTION_PACKAGE_ATTRS = SimpleNamespace(
    ARCH="arch",
    EPOCH="epoch",
    FILENAME="filename",
    NAME="name",
    REBOOT_SUGGESTED="reboot_suggested",
    RELOGIN_SUGGESTED="relogin_suggested",
    RESTART_SUGGESTED="restart_suggested",
    RELEASE="release",
    SRC="src",
    SUM="sum",
    SUM_TYPE="sum_type",
    VERSION="version",
)

CR_UPDATE_REFERENCE_ATTRS = SimpleNamespace(HREF="href", ID="id", TITLE="title", TYPE="type")

CR_UPDATE_COLLECTION_ATTRS_MODULE = SimpleNamespace(
    NAME="name", STREAM="stream", VERSION="version", CONTEXT="context", ARCH="arch"
)

PULP_UPDATE_REFERENCE_ATTRS = SimpleNamespace(
    HREF="href", ID="ref_id", TITLE="title", TYPE="ref_type"
)

PULP_PACKAGE_ATTRS = CR_PACKAGE_ATTRS
PULP_UPDATE_RECORD_ATTRS = CR_UPDATE_RECORD_ATTRS
PULP_UPDATE_COLLECTION_ATTRS = CR_UPDATE_COLLECTION_ATTRS
PULP_UPDATE_COLLECTION_PACKAGE_ATTRS = CR_UPDATE_COLLECTION_PACKAGE_ATTRS

PULP_UPDATE_COLLECTION_ATTRS_MODULE = CR_UPDATE_COLLECTION_ATTRS_MODULE

MODULEMD_MODULE_ATTR = SimpleNamespace(
    ARCH="arch",
    ARTIFACTS="artifacts",
    STATIC_CONTEXT="static_context",
    CONTEXT="context",
    NAME="name",
    STREAM="stream",
    VERSION="version",
    DEPENDENCIES="dependencies",
)

MODULEMD_MODULEDEFAULTS_ATTR = SimpleNamespace(
    MODULE="module", STREAM="stream", PROFILES="profiles"
)

PULP_MODULEDEFAULTS_ATTR = SimpleNamespace(
    MODULE="module", STREAM="stream", PROFILES="profiles", DIGEST="digest"
)

PULP_MODULE_ATTR = MODULEMD_MODULE_ATTR

LIBCOMPS_GROUP_ATTRS = SimpleNamespace(
    ID="id",
    DEFAULT="default",
    USER_VISIBLE="uservisible",
    DISPLAY_ORDER="display_order",
    NAME="name",
    DESCRIPTION="desc",
    PACKAGES="packages",
    BIARCH_ONLY="biarchonly",
    DESC_BY_LANG="desc_by_lang",
    NAME_BY_LANG="name_by_lang",
)

LIBCOMPS_CATEGORY_ATTRS = SimpleNamespace(
    ID="id",
    NAME="name",
    DESCRIPTION="desc",
    DISPLAY_ORDER="display_order",
    GROUP_IDS="group_ids",
    DESC_BY_LANG="desc_by_lang",
    NAME_BY_LANG="name_by_lang",
)

LIBCOMPS_ENVIRONMENT_ATTRS = SimpleNamespace(
    ID="id",
    NAME="name",
    DESCRIPTION="desc",
    DISPLAY_ORDER="display_order",
    GROUP_IDS="group_ids",
    OPTION_IDS="option_ids",
    DESC_BY_LANG="desc_by_lang",
    NAME_BY_LANG="name_by_lang",
)

PULP_LANGPACKS_ATTRS = SimpleNamespace(MATCHES="matches")

PULP_GROUP_ATTRS = SimpleNamespace(
    ID="id",
    DEFAULT="default",
    USER_VISIBLE="user_visible",
    DISPLAY_ORDER="display_order",
    NAME="name",
    DESCRIPTION="description",
    PACKAGES="packages",
    BIARCH_ONLY="biarch_only",
    DESC_BY_LANG="desc_by_lang",
    NAME_BY_LANG="name_by_lang",
)

PULP_CATEGORY_ATTRS = SimpleNamespace(
    ID="id",
    NAME="name",
    DESCRIPTION="description",
    DISPLAY_ORDER="display_order",
    GROUP_IDS="group_ids",
    DESC_BY_LANG="desc_by_lang",
    NAME_BY_LANG="name_by_lang",
)

PULP_ENVIRONMENT_ATTRS = SimpleNamespace(
    ID="id",
    NAME="name",
    DESCRIPTION="description",
    DISPLAY_ORDER="display_order",
    GROUP_IDS="group_ids",
    OPTION_IDS="option_ids",
    DESC_BY_LANG="desc_by_lang",
    NAME_BY_LANG="name_by_lang",
)

PACKAGES_DIRECTORY = "Packages"
DIST_TREE_MAIN_REPO_PATH = "."

# Mappings of the possible integer values of "sum_type" on Advisory packages to their user-facing
# string representation. Should mirror the createrepo_c source code:
# https://github.com/rpm-software-management/createrepo_c/blob/master/src/checksum.h#L43-L54
# Please keep dict consistent with createrepo_c mapping mentioned above.
ADVISORY_SUM_TYPE_TO_NAME = {
    0: "unknown",
    1: "md5",
    2: "sha",
    3: "sha1",
    4: "sha224",
    5: "sha256",
    6: "sha384",
    7: "sha512",
}



from .advisory import (  # noqa
    UpdateCollection,
    UpdateCollectionPackage,
    UpdateRecord,
    UpdateReference,
)
from .comps import PackageCategory, PackageEnvironment, PackageGroup, PackageLangpacks  # noqa
from .custom_metadata import RepoMetadataFile  # noqa
from .distribution import Addon, Checksum, DistributionTree, Image, Variant  # noqa
from .modulemd import Modulemd, ModulemdDefaults  # noqa
from .package import Package  # noqa
from .repository import RpmDistribution, RpmPublication, RpmRemote, UlnRemote, RpmRepository  # noqa



import requests

from pulp_smash import config
from pulp_smash.pulp3.bindings import PulpTestCase, monitor_task
from pulp_smash.pulp3.utils import (
    delete_orphans,
    gen_distribution,
    gen_repo,
)

from pulp_rpm.tests.functional.utils import gen_rpm_client

from pulpcore.client.pulp_rpm import (
    DistributionsRpmApi,
    RepositoriesRpmApi,
    PublicationsRpmApi,
    RpmRpmPublication,
)


class ContentHandlerTests(PulpTestCase):
    """Whether the RpmDistribution.content_handler* methods work."""

    @classmethod
    def tearDownClass(cls) -> None:
        """Clean up after testing."""
        delete_orphans(cls.cfg)

    @classmethod
    def setUpClass(cls) -> None:
        """Set up the class."""
        cls.cfg = config.get_config()
        cls.client = gen_rpm_client()
        cls.repo_api = RepositoriesRpmApi(cls.client)
        cls.publications_api = PublicationsRpmApi(cls.client)
        cls.distributions_api = DistributionsRpmApi(cls.client)
        delete_orphans(cls.cfg)

    def setUp(self) -> None:
        """Set up the test."""
        repo = self.repo_api.create(gen_repo())
        self.addCleanup(self.repo_api.delete, repo.pulp_href)

        publish_data = RpmRpmPublication(repository=repo.pulp_href)
        publish_response = self.publications_api.create(publish_data)
        created_resources = monitor_task(publish_response.task).created_resources
        publication_href = created_resources[0]
        self.addCleanup(self.publications_api.delete, publication_href)

        dist_data = gen_distribution(publication=publication_href)
        dist_response = self.distributions_api.create(dist_data)
        created_resources = monitor_task(dist_response.task).created_resources
        self.dist = self.distributions_api.read(created_resources[0])
        self.addCleanup(self.distributions_api.delete, self.dist.pulp_href)

    def testConfigRepoInListingUnsigned(self):
        """Whether the served resources are in the directory listing."""
        resp = requests.get(self.dist.base_url)

        self.assertEqual(resp.status_code, 200)
        self.assertIn(b"config.repo", resp.content)
        self.assertNotIn(b"public.key", resp.content)

    def testConfigRepoUnsigned(self):
        """Whether config.repo can be downloaded and has the right content."""
        resp = requests.get(f"{self.dist.base_url}config.repo")

        self.assertEqual(resp.status_code, 200)
        self.assertIn(bytes(f"[{self.dist.name}]\n", "utf-8"), resp.content)
        self.assertIn(bytes(f"baseurl={self.dist.base_url}\n", "utf-8"), resp.content)
        self.assertIn(bytes("gpgcheck=0\n", "utf-8"), resp.content)
        self.assertIn(bytes("repo_gpgcheck=0", "utf-8"), resp.content)



"""Tests for the retention policy feature of repositories."""

from collections import defaultdict

from pulp_smash import config
from pulp_smash.pulp3.bindings import PulpTestCase, monitor_task
from pulp_smash.pulp3.utils import (
    gen_repo,
    get_added_content_summary,
    get_content,
    get_content_summary,
    get_removed_content,
    get_removed_content_summary,
    delete_orphans,
)

from pulp_rpm.tests.functional.constants import (
    PULP_TYPE_PACKAGE,
    RPM_FIXTURE_SUMMARY,
)
from pulp_rpm.tests.functional.utils import (
    gen_rpm_client,
    gen_rpm_remote,
    progress_reports,
)
from pulp_rpm.tests.functional.utils import set_up_module as setUpModule  # noqa:F401

from pulpcore.client.pulp_rpm import (
    RepositoriesRpmApi,
    RpmRepositorySyncURL,
    RemotesRpmApi,
)
from pulpcore.client.pulp_rpm.exceptions import ApiException


class RetentionPolicyTestCase(PulpTestCase):
    """Verify functionality of the "retain_package_versions" setting on a Repository."""

    @classmethod
    def setUpClass(cls):
        """Create class-wide variables."""
        cls.cfg = config.get_config()
        cls.client = gen_rpm_client()
        cls.repo_api = RepositoriesRpmApi(cls.client)
        cls.remote_api = RemotesRpmApi(cls.client)
        delete_orphans(cls.cfg)

    def test_sync_with_retention(self):
        """Verify functionality with sync.

        Do the following:

        1. Create a repository, and a remote.
        2. Sync the remote.
        3. Assert that the correct number of units were added and are present
           in the repo.
        4. Change the "retain_package_versions" on the repository to 1 (retain the latest
           version only).
        5. Sync the remote one more time.
        6. Assert that repository version is different from the previous one.
        7. Assert the repository version we end with has only one version of each package.
        """
        repo = self.repo_api.create(gen_repo())
        self.addCleanup(self.repo_api.delete, repo.pulp_href)

        remote = self.remote_api.create(gen_rpm_remote())
        self.addCleanup(self.remote_api.delete, remote.pulp_href)

        self.sync(repository=repo, remote=remote, optimize=False)
        repo = self.repo_api.read(repo.pulp_href)

        # Test that, by default, everything is retained / nothing is tossed out.
        self.assertDictEqual(get_content_summary(repo.to_dict()), RPM_FIXTURE_SUMMARY)
        self.assertDictEqual(get_added_content_summary(repo.to_dict()), RPM_FIXTURE_SUMMARY)

        # Set the retention policy to retain only 1 version of each package
        repo_data = repo.to_dict()
        repo_data.update({"retain_package_versions": 1})
        self.repo_api.update(repo.pulp_href, repo_data)
        repo = self.repo_api.read(repo.pulp_href)

        self.sync(repository=repo, remote=remote, optimize=False)
        repo = self.repo_api.read(repo.pulp_href)

        # Test that only one version of each package is present
        self.assertTrue(
            self.check_retention_policy(get_content(repo.to_dict())[PULP_TYPE_PACKAGE], 1)
        )
        # Test that (only) 4 RPMs were removed (no advisories etc. touched)
        self.assertDictEqual(get_removed_content_summary(repo.to_dict()), {PULP_TYPE_PACKAGE: 4})
        # Test that the versions that were removed are the versions we expect.
        versions_for_packages = self.versions_for_packages(
            get_removed_content(repo.to_dict())[PULP_TYPE_PACKAGE]
        )
        self.assertDictEqual(
            versions_for_packages,
            {"duck": ["0.6", "0.7"], "kangaroo": ["0.2"], "walrus": ["0.71"]},
            versions_for_packages,
        )
        # TODO: Test that modular RPMs unaffected?

    def test_mirror_sync_with_retention_fails(self):
        """Verify functionality with sync.

        Do the following:

        1. Create a repository with 'retain_package_versions' set, and a remote.
        2. Sync the remote in mirror mode.
        3. Assert that the sync fails.
        """
        repo_data = gen_repo()
        repo_data.update({"retain_package_versions": 1})
        repo = self.repo_api.create(repo_data)
        self.addCleanup(self.repo_api.delete, repo.pulp_href)

        remote = self.remote_api.create(gen_rpm_remote())
        self.addCleanup(self.remote_api.delete, remote.pulp_href)

        with self.assertRaises(ApiException) as exc:
            self.sync(repository=repo, remote=remote, optimize=False, mirror=True)
            self.assertEqual(exc.code, 400)

    def versions_for_packages(self, packages):
        """Get a list of versions for each package present in a list of Package dicts.

        Args:
            packages: List of Package info dicts
        """
        packages_by_version = defaultdict(list)

        for package in packages:
            packages_by_version[package["name"]].append(package["version"])

        for pkg_list in packages_by_version.values():
            pkg_list.sort()

        return packages_by_version

    def check_retention_policy(self, packages, retain_package_versions):
        """Check that the number of versions of each package <= permitted number.

        Args:
            packages: List of Package info dicts
            retention_policy: Number of package versions permitted.
        """
        return all(
            [
                len(versions) <= retain_package_versions
                for versions in self.versions_for_packages(packages).values()
            ]
        )

    def sync(self, repository, remote, optimize=True, mirror=False):
        """Sync a repository and return the task.

        Args:
            repository (pulp_rpm.app.models.repository.RpmRepository):
                object of RPM repository
            remote (pulp_rpm.app.models.repository.RpmRemote):
                object of RPM Remote
            optimize (bool):
                whether to enable optimized sync
            mirror (bool):
                whether to use mirror-mode sync
        Returns (list):
            list of the ProgressReport objects created from this sync
        """
        repository_sync_data = RpmRepositorySyncURL(
            remote=remote.pulp_href, optimize=optimize, mirror=mirror
        )
        sync_response = self.repo_api.sync(repository.pulp_href, repository_sync_data)
        monitor_task(sync_response.task)
        return progress_reports(sync_response.task)



import urllib.parse
import textwrap

from gettext import gettext as _
from logging import getLogger

from aiohttp.web_response import Response
from django.conf import settings
from django.contrib.postgres.fields import JSONField
from django.db import models, transaction
from pulpcore.plugin.download import DownloaderFactory
from pulpcore.plugin.models import (
    Artifact,
    AsciiArmoredDetachedSigningService,
    Content,
    CreatedResource,
    Remote,
    Repository,
    RepositoryVersion,
    Publication,
    Distribution,
    Task,
)
from pulpcore.plugin.repo_version_utils import remove_duplicates, validate_repo_version

from pulp_rpm.app.constants import CHECKSUM_CHOICES, CHECKSUM_TYPES
from pulp_rpm.app.models import (
    DistributionTree,
    Package,
    PackageCategory,
    PackageGroup,
    PackageEnvironment,
    PackageLangpacks,
    RepoMetadataFile,
    Modulemd,
    ModulemdDefaults,
    UpdateRecord,
)

from pulp_rpm.app.downloaders import RpmDownloader, RpmFileDownloader, UlnDownloader
from pulp_rpm.app.exceptions import DistributionTreeConflict
from pulp_rpm.app.shared_utils import urlpath_sanitize

log = getLogger(__name__)


class RpmRemote(Remote):
    """
    Remote for "rpm" content.
    """

    TYPE = "rpm"
    sles_auth_token = models.CharField(max_length=512, null=True)

    @property
    def download_factory(self):
        """
        Return the DownloaderFactory which can be used to generate asyncio capable downloaders.

        Returns:
            DownloadFactory: The instantiated DownloaderFactory to be used by
                get_downloader()

        """
        try:
            return self._download_factory
        except AttributeError:
            self._download_factory = DownloaderFactory(
                self,
                downloader_overrides={
                    "http": RpmDownloader,
                    "https": RpmDownloader,
                    "file": RpmFileDownloader,
                },
            )
            return self._download_factory

    def get_downloader(self, remote_artifact=None, url=None, **kwargs):
        """
        Get a downloader from either a RemoteArtifact or URL that is configured with this Remote.

        This method accepts either `remote_artifact` or `url` but not both. At least one is
        required. If neither or both are passed a ValueError is raised.

        Args:
            remote_artifact (:class:`~pulpcore.app.models.RemoteArtifact`): The RemoteArtifact to
                download.
            url (str): The URL to download.
            kwargs (dict): This accepts the parameters of
                :class:`~pulpcore.plugin.download.BaseDownloader`.
        Raises:
            ValueError: If neither remote_artifact and url are passed, or if both are passed.
        Returns:
            subclass of :class:`~pulpcore.plugin.download.BaseDownloader`: A downloader that
            is configured with the remote settings.

        """
        if self.sles_auth_token:
            kwargs["sles_auth_token"] = self.sles_auth_token
        return super().get_downloader(remote_artifact=remote_artifact, url=url, **kwargs)

    class Meta:
        default_related_name = "%(app_label)s_%(model_name)s"


class UlnRemote(Remote):
    """
    Remote for "uln" content.
    """

    TYPE = "uln"
    uln_server_base_url = models.CharField(max_length=512, null=True)

    @property
    def download_factory(self):
        """
        Return the DownloaderFactory which can be used to generate asyncio capable downloaders.

        Returns:
            DownloadFactory: The instantiated DownloaderFactory to be used by
                get_downloader()

        """
        try:
            return self._download_factory
        except AttributeError:
            self._download_factory = DownloaderFactory(
                self,
                downloader_overrides={
                    "uln": UlnDownloader,
                },
            )
            self._download_factory._handler_map["uln"] = self._download_factory._http_or_https
            return self._download_factory

    def get_downloader(self, remote_artifact=None, url=None, **kwargs):
        """
        Get a downloader from either a RemoteArtifact or URL that is configured with this Remote.

        This method accepts either `remote_artifact` or `url` but not both. At least one is
        required. If neither or both are passed a ValueError is raised.

        Args:
            remote_artifact (:class:`~pulpcore.app.models.RemoteArtifact`): The RemoteArtifact to
                download.
            url (str): The URL to download. Can be a ULN url.
            kwargs (dict): This accepts the parameters of
                :class:`~pulpcore.plugin.download.BaseDownloader`.
        Raises:
            ValueError: If neither remote_artifact and url are passed, or if both are passed.
        Returns:
            subclass of :class:`~pulpcore.plugin.download.BaseDownloader`: A downloader that
            is configured with the remote settings.

        """
        if self.uln_server_base_url:
            uln_server_base_url = self.uln_server_base_url
        else:
            uln_server_base_url = settings.DEFAULT_ULN_SERVER_BASE_URL

        return super().get_downloader(
            remote_artifact=remote_artifact,
            url=url,
            username=self.username,
            password=self.password,
            uln_server_base_url=uln_server_base_url,
            **kwargs,
        )

    class Meta:
        default_related_name = "%(app_label)s_%(model_name)s"


class RpmRepository(Repository):
    """
    Repository for "rpm" content.

    Fields:

        sub_repo (Boolean):
            Whether is sub_repo or not
        last_sync_revision_number (Text):
            The revision number
        last_sync_remote (Remote):
            The remote used for the last sync
        last_sync_repo_version (Integer):
            The repo version number of the last sync
        last_sync_repomd_checksum (Text):
            The repo version repomd.xml file sha256
        original_checksum_types (JSON):
            Checksum for each metadata type
    """

    TYPE = "rpm"
    CONTENT_TYPES = [
        Package,
        UpdateRecord,
        PackageCategory,
        PackageGroup,
        PackageEnvironment,
        PackageLangpacks,
        RepoMetadataFile,
        DistributionTree,
        Modulemd,
        ModulemdDefaults,
    ]
    REMOTE_TYPES = [RpmRemote, UlnRemote]
    GPGCHECK_CHOICES = [(0, 0), (1, 1)]

    metadata_signing_service = models.ForeignKey(
        AsciiArmoredDetachedSigningService, on_delete=models.SET_NULL, null=True
    )
    sub_repo = models.BooleanField(default=False)
    last_sync_revision_number = models.CharField(max_length=20, null=True)
    last_sync_remote = models.ForeignKey(Remote, null=True, on_delete=models.SET_NULL)
    last_sync_repo_version = models.PositiveIntegerField(default=0)
    last_sync_repomd_checksum = models.CharField(max_length=64, null=True)
    original_checksum_types = JSONField(default=dict)
    retain_package_versions = models.PositiveIntegerField(default=0)

    autopublish = models.BooleanField(default=False)
    metadata_checksum_type = models.CharField(
        default=CHECKSUM_TYPES.SHA256, choices=CHECKSUM_CHOICES, max_length=10
    )
    package_checksum_type = models.CharField(
        default=CHECKSUM_TYPES.SHA256, choices=CHECKSUM_CHOICES, max_length=10
    )
    gpgcheck = models.IntegerField(default=0, choices=GPGCHECK_CHOICES)
    repo_gpgcheck = models.IntegerField(default=0, choices=GPGCHECK_CHOICES)
    sqlite_metadata = models.BooleanField(default=False)

    def new_version(self, base_version=None):
        """
        Create a new RepositoryVersion for this Repository.

        Creation of a RepositoryVersion should be done in a RQ Job.

        Args:
            repository (pulpcore.app.models.Repository): to create a new version of
            base_version (pulpcore.app.models.RepositoryVersion): an optional repository version
                whose content will be used as the set of content for the new version

        Returns:
            pulpcore.app.models.RepositoryVersion: The Created RepositoryVersion

        """
        with transaction.atomic():
            latest_version = self.versions.latest()
            if not latest_version.complete:
                latest_version.delete()

            version = RepositoryVersion(
                repository=self, number=int(self.next_version), base_version=base_version
            )
            version.save()

            if base_version:
                # first remove the content that isn't in the base version
                version.remove_content(version.content.exclude(pk__in=base_version.content))
                # now add any content that's in the base_version but not in version
                version.add_content(base_version.content.exclude(pk__in=version.content))

            if Task.current() and not self.sub_repo:
                resource = CreatedResource(content_object=version)
                resource.save()
            return version

    def on_new_version(self, version):
        """
        Called when new repository versions are created.

        Args:
            version: The new repository version.
        """
        super().on_new_version(version)

        # avoid circular import issues
        from pulp_rpm.app import tasks

        if self.autopublish:
            publication = tasks.publish(
                repository_version_pk=version.pk,
                gpgcheck_options={"gpgcheck": self.gpgcheck, "repo_gpgcheck": self.repo_gpgcheck},
                metadata_signing_service=self.metadata_signing_service,
                checksum_types={
                    "metadata": self.metadata_checksum_type,
                    "package": self.package_checksum_type,
                },
                sqlite_metadata=self.sqlite_metadata,
            )
            distributions = self.distributions.all()

            if publication and distributions:
                for distribution in distributions:
                    distribution.publication = publication
                    distribution.save()

    @staticmethod
    def artifacts_for_version(version):
        """
        Return the artifacts for an RpmRepository version.

        Override the default behavior to include DistributionTree artifacts from nested repos.

        Args:
            version (pulpcore.app.models.RepositoryVersion): to get the artifacts for

        Returns:
            django.db.models.QuerySet: The artifacts that are contained within this version.

        """
        qs = Artifact.objects.filter(content__pk__in=version.content)
        for tree in DistributionTree.objects.filter(pk__in=version.content):
            qs |= tree.artifacts()

        return qs

    class Meta:
        default_related_name = "%(app_label)s_%(model_name)s"

    def finalize_new_version(self, new_version):
        """
        Ensure there are no duplicates in a repo version and content is not broken.

        Remove duplicates based on repo_key_fields.
        Ensure that modulemd is added with all its RPMs.
        Ensure that modulemd is removed with all its RPMs.
        Resolve advisory conflicts when there is more than one advisory with the same id.

        Args:
            new_version (pulpcore.app.models.RepositoryVersion): The incomplete RepositoryVersion
                to finalize.
        """
        if new_version.base_version:
            previous_version = new_version.base_version
        else:
            try:
                previous_version = new_version.previous()
            except RepositoryVersion.DoesNotExist:
                previous_version = None

        remove_duplicates(new_version)
        self._resolve_distribution_trees(new_version, previous_version)

        from pulp_rpm.app.modulemd import resolve_module_packages  # avoid circular import

        resolve_module_packages(new_version, previous_version)

        self._apply_retention_policy(new_version)

        from pulp_rpm.app.advisory import resolve_advisories  # avoid circular import

        resolve_advisories(new_version, previous_version)
        validate_repo_version(new_version)

    def _apply_retention_policy(self, new_version):
        """Apply the repository's "retain_package_versions" settings to the new version.

        Remove all non-modular packages that are older than the retention policy. A value of 0
        for the package retention policy represents disabled. A value of 3 would mean that the
        3 most recent versions of each package would be kept while older versions are discarded.

        Args:
            new_version (models.RepositoryVersion): Repository version to filter
        """
        assert (
            not new_version.complete
        ), "Cannot apply retention policy to completed repository versions"

        if self.retain_package_versions > 0:
            # It would be more ideal if, instead of annotating with an age and filtering manually,
            # we could use Django to filter the particular Package content we want to delete.
            # Something like ".filter(F('age') > self.retain_package_versions)" would be better
            # however this is not currently possible with Django. It would be possible with raw
            # SQL but the repository version content membership subquery is currently
            # django-managed and would be difficult to share.
            #
            # Instead we have to do the filtering manually.
            nonmodular_packages = (
                Package.objects.with_age()
                .filter(
                    pk__in=new_version.content.filter(pulp_type=Package.get_pulp_type()),
                    is_modular=False,  # don't want to filter out modular RPMs
                )
                .only("pk")
            )

            old_packages = []
            for package in nonmodular_packages:
                if package.age > self.retain_package_versions:
                    old_packages.append(package.pk)

            new_version.remove_content(Content.objects.filter(pk__in=old_packages))

    def _resolve_distribution_trees(self, new_version, previous_version):
        """
        There can be only one distribution tree in a repo version.

        Args:
            version (pulpcore.app.models.RepositoryVersion): current incomplete repository version
            previous_version (pulpcore.app.models.RepositoryVersion):  a version preceding
                                                                       the current incomplete one
        """
        disttree_pulp_type = DistributionTree.get_pulp_type()
        current_disttrees = new_version.content.filter(pulp_type=disttree_pulp_type)

        if len(current_disttrees) < 2:
            return

        if previous_version:
            previous_disttree = previous_version.content.get(pulp_type=disttree_pulp_type)
            new_version.remove_content(Content.objects.filter(pk=previous_disttree.pk))

        incoming_disttrees = new_version.content.filter(pulp_type=disttree_pulp_type)
        if len(incoming_disttrees) != 1:
            raise DistributionTreeConflict(
                _("More than one distribution tree cannot be added to a " "repository version.")
            )


class RpmPublication(Publication):
    """
    Publication for "rpm" content.
    """

    GPGCHECK_CHOICES = [(0, 0), (1, 1)]

    TYPE = "rpm"
    metadata_checksum_type = models.CharField(choices=CHECKSUM_CHOICES, max_length=10)
    package_checksum_type = models.CharField(choices=CHECKSUM_CHOICES, max_length=10)
    gpgcheck = models.IntegerField(default=0, choices=GPGCHECK_CHOICES)
    repo_gpgcheck = models.IntegerField(default=0, choices=GPGCHECK_CHOICES)
    sqlite_metadata = models.BooleanField(default=False)

    class Meta:
        default_related_name = "%(app_label)s_%(model_name)s"


class RpmDistribution(Distribution):
    """
    Distribution for "rpm" content.
    """

    TYPE = "rpm"
    repository_config_file_name = "config.repo"

    def content_handler(self, path):
        """Serve config.repo and public.key."""
        if path == self.repository_config_file_name:
            base_url = f"{settings.CONTENT_ORIGIN}{settings.CONTENT_PATH_PREFIX}{self.base_path}/"
            publication = self.publication.cast()
            val = textwrap.dedent(
                f"""\
                [{self.name}]
                enabled=1
                baseurl={base_url}
                gpgcheck={publication.gpgcheck}
                repo_gpgcheck={publication.repo_gpgcheck}
                """
            )

            repository_pk = self.publication.repository.pk
            repository = RpmRepository.objects.get(pk=repository_pk)
            signing_service = repository.metadata_signing_service
            if signing_service:
                gpgkey_path = urlpath_sanitize(
                    settings.CONTENT_ORIGIN, settings.CONTENT_PATH_PREFIX
                )
                gpgkey_path = urllib.parse.urljoin(gpgkey_path, self.base_path, True)
                gpgkey_path += "/repodata/public.key"

                val += f"gpgkey={gpgkey_path}\n"

            return Response(body=val)

    def content_handler_list_directory(self, rel_path):
        """Return the extra dir entries."""
        retval = set()
        if rel_path == "":
            retval.add(self.repository_config_file_name)
        return retval

    class Meta:
        default_related_name = "%(app_label)s_%(model_name)s"
